

hyperlink=function(screenNames){
  linkList=paste('\'<a href= "https://twitter.com/',screenNames,'" target="_blank"> ',screenNames,'</a>\'',sep='')
  return(linkList)}


TrainTest_SetBuilder=function(data,PositiveLabels,NegativeLabels,p.test.split){ 
  
  data$creation_date=normaLize(as.numeric(data$creation_date))
  data$lastTweet=normaLize(as.numeric(data$lastTweet))
  normvec=c("followingCount","followerCount","tweetCount","tweetsCollected","FinDegree","FoutDegree","FiP","MinDegree","MoutDegree","HTcount")
  data[,normvec]=sapply(normvec, function(x) normaLize(data[,x]))
  
  pos=data[data$userID %in% PositiveLabels,]
  class=array(1,dim=dim(pos)[1])
  pos=cbind(class,pos)
  pos.test.ind=sample(1:dim(pos)[1],floor(dim(pos)[1]*p.test.split))
  pos.test=pos[pos.test.ind,]
  pos.train=pos[as.logical(1- (1:dim(pos)[1]) %in% pos.test.ind    ),]
  
  neg=data[data$userID %in% NegativeLabels,]
  class=array(0,dim=dim(neg)[1])
  neg=cbind(class,neg)
  neg.test.ind=sample(1:dim(neg)[1],floor(dim(neg)[1]*p.test.split))
  neg.test=neg[neg.test.ind,]
  neg.train=neg[as.logical(1- (1:dim(neg)[1]) %in% neg.test.ind    ),]
  
  d.train=rbind(pos.train,neg.train)
  d.test=rbind(pos.test,neg.test)
  d.predict=data[as.logical(1-(data$ScreenName %in% c(d.train$ScreenName,d.test$ScreenName))),]
  return(list(d.train,d.test,d.predict))}


classifier=function(data_list,
                    algorithm,# 'svmRadial'  # 'lssvmPoly' 'lssvmLinear 'JRip' 'J48' 
                    evaluation=TRUE,
                    metric,# 'Kappa'
                    ratio=c(.2,.8),
                    t){ 
  feature_set=read.csv(file.path('input','featureSetNames.csv'),stringsAsFactors=FALSE)
  feature_set=feature_set$feature[as.logical(feature_set$selection)]  

  data_list[[1]]$class=as.factor(data_list[[1]]$class)
  data_list[[2]]$class=as.factor(data_list[[2]]$class)
  temp.features=c('class',feature_set)
  if(evaluation==TRUE)  train.data=data_list[[1]][,temp.features]
  if(evaluation==FALSE) train.data=rbind(data_list[[1]][,temp.features],data_list[[2]][,temp.features])
  
  if(evaluation==TRUE)  result.data=data_list[[2]][,temp.features]
  if(evaluation==FALSE) result.data=data_list[[3]][,temp.features[-1]]
  
  if(algorithm != 'randomForest') classifier=train(class ~ .,data = train.data,na.action=na.omit,method=algorithm,metric='Kappa')
  if(algorithm == 'randomForest') classifier=randomForest(class ~ .,data = train.data,na.action=na.omit,cutoff=c(ratio,1-ratio))
  
  if(evaluation==TRUE & algorithm != 'randomForest') test=predict.train(classifier,newdata=result.data,type='raw')
  if(evaluation==TRUE & algorithm == 'randomForest') test=predict(classifier,newdata=result.data,type='response')
  if(evaluation==TRUE) model.obj=confusionMatrix(test,data_list[[2]]$class)
  if(evaluation==TRUE){capture.output(cat(paste('ALGORITHM: ',algorithm,sep=''),'\n'),cat('\n'),
                                      cat(paste('FEATURE SET: ',paste(feature_set,collapse=', '),sep=''),'\n'),
                                      cat('\n'),cat(paste('TIME: ',as.character(Sys.time()),sep=''),'\n'),
                                      cat('\n'),model.obj,file=file.path('output',paste(t,'_model_eval.txt',sep='')))}
  if(evaluation==TRUE) return(model.obj)
  
  if(evaluation==FALSE & algorithm == 'randomForest' )  predicted=predict(classifier,newdata=result.data,type='response')
  if(evaluation==FALSE & algorithm != 'randomForest' )  predicted=predict.train(classifier,newdata=result.data,type='raw')
  if(evaluation==FALSE){                                
    predicted.ids=data_list[[3]]$userID[predicted==1]
    positive.sample=iconv(enc2utf8(sample(data_list[[3]]$ScreenName[predicted==1],15)),sub='bytes')
    negative.sample=iconv(enc2utf8(sample(data_list[[3]]$ScreenName[predicted==0],15)),sub='bytes')
    df=c('IVCC Classifier Report <br>',paste('Classifier : ',algorithm,', \n with winning class ratio = ',r,'<br>',sep=''),paste(sum(as.numeric(predicted)-1),' predicted positive case<br> ',dim(data_list[[3]])[1]-sum(as.numeric(predicted)-1),' predicted negative case<br><br>',sep=''),
         '','Sample of Positive Cases <br>',
         paste('<a href= "https://twitter.com/',positive.sample,'" target="_blank"> ',positive.sample,'</a><br>',sep=''),
         '',"Sample of Negative Cases<br>",
         paste('<a href= "https://twitter.com/',negative.sample,'" target="_blank"> ',negative.sample,'</a><br>',sep=''),'</body>')
    
    target <- HTMLInitFile(file.path('output'),filename=paste(t,'_Model_Validation',sep=''),
                           Title=paste(t,'_Model_Validation',sep=''),CSSFile=file.path('IVCC_css','responsive.css'))
    R2HTML::HTML(df,classfirstline='header',file=target,append=TRUE, Border=0, classtable='tablesorter',sortableDF=TRUE)
    HTMLEndFile()
  }
  if(evaluation==FALSE) return(predicted.ids)
  
}

normaLize=function(t){
  trange=range(t)
  r=trange[2]-trange[1]
  t=2*t/r
  trange=range(t)
  t=t-trange[1]-1
  return(t)}

barPlotBuilder=function(userData,
                        targetIDs,
                        clusterType,
                        sort_type,
                        top_n){
  if(length(targetIDs)>0) targetIDs=fread(targetIDs,data.table=FALSE,integer64='numeric')$userID
  if(clusterType=='mention') clusterType='RMLouvain'
  if(clusterType=='hashtag') clusterType='topic_cluster'
  if(clusterType=='follow') clusterType='RFLouvain'
  if(clusterType=='dense') clusterType='denseCluster'
  
  cSizeFunction=function(x,tempVec){
    s=sum(tempVec==x,na.rm=TRUE) 
    return(s)
  }

  tSizeFunction=function(x,temp,clusterType){
    
    s=sum(temp$targetIDs[temp[,clusterType]==x],na.rm=TRUE)
    return(s)
  }
  
  
  temp=data.frame(userData,targetIDs=userData$userID %in% targetIDs)
  
  cSeries=sort(unique(temp[,clusterType]))
  cSize=sapply(cSeries,cSizeFunction,temp[,clusterType])
  tSize=sapply(cSeries,tSizeFunction,temp,clusterType)
  ntSize=cSize-tSize
  barDF=data.frame(cluster=cSeries,seeds=tSize,non_seeds=ntSize,seed_proportion=tSize/cSize)
  barDF=barDF[order(barDF[,sort_type],decreasing=TRUE),]
  barDF=barDF[1:top_n,]
  barDF=barDF[order(barDF[,sort_type]),]
  barplot(height=as.matrix(t(barDF[,3:2])),horiz=TRUE,las=2,names.arg=barDF[,1],main=paste('Clusters of Interest\n cluster type: ',clusterType,'\n sort type: ',ifelse(sort_type=='seeds','seed count','seed proportion'),sep=''),ylab='cluster',xlab='size')
  
  
}
