
shinyUI(navbarPage(
  title="Extremist Community Analysis",

  tabPanel('Data Summary', 
           fluidPage(
            titlePanel('IVCC Project Overview'),
            sidebarLayout(
                sidebarPanel(
                  p('The project has been developed at the CASOS(insert link) at Carnegie Mellon University.  It is meant to be a proof of concept for Iterative Vertex Clustering and Classification (IVCC), as a means to detect threat groups operating in social media.'),
                  strong("Instructions: "),
                  p('this website is designed to enable a user to explore a large dataset of potential members of an online extremist community.  For illustrative purposes we will use the Euromaidan community on Twitter, though we do not claim this meets the criteria of an extremist community, their levels of online activism make the group similar structurally.'), 
                radioButtons("dataType", "Choose Output Type:",
                             c("Dense Features" = 1,
                               "Final Output" = 2)),  
                strong("Barplot Controls: "),
                selectInput("clusterType", "Choose the cluster type you wish to analyze:",
                            choices = c('dense','hashtag','mention','follow')),
                radioButtons("sort_type", "Sort By:",
                               c("Proportion of Users Manually Labelled Jihadist" = "seed_proportion",
                                 "Number of Users Manually Labelled Jihadist" = "seeds")),
                numericInput('top_n', 'Display the top n clusters', 10, min = 2, max = 20, step = 1),
                strong("Ways to Support the Project: "),
                p("Provide Feedback: This dataset is extremely rich and incorporates language and geolocation features.  An understanding of useful additions could help prioritize future research."),
                p("Provide User Labels: Identifying accounts that are clearly Jihadist or apparently not Jihadist helps us continue to train and adjust the algorithms used in IVCC.  If you would like to provide labelled data visit here INSERT LINK"),
                img(src='logos.png',width=450)
                ),
                mainPanel(
                  plotOutput("barplot")
                  )
              )
           )
           
           ),
 
  tabPanel("Dense Clusters by User Language",
           fluidPage(
             fluidPage(
               # Application title
               titlePanel("Dense Clusters by Language Freq"),

                 # Sidebar with a slider and selection inputs 
                 mainPanel(
                   dataTableOutput('outputClusters')
                 )
               )
             )
           
  ),
  
  
  
  tabPanel('Dense Communities',
           fluidPage(
             fluidPage(
               # Application title
               titlePanel("Topic Analysis by Hashtag"),
               sidebarLayout(
                 # Sidebar with a slider and selection inputs
                 sidebarPanel(
                   numericInput("tweetCount","Minimum Tweet Count:",0),
                   numericInput("followCount","Minimum Follower Count:",0),               
                   br(),
                   p('This page is designed to enable an analyst to identify topic clusters of interest by displaying the most frequent hashtags used by the clusters members over the past year.  First select a cluster, and then adjust the number of words and font size.  Additionally, you can select wether you would like to see the tags in their original language or translated.'),
                   img(src='logos.png',width=450)
                 ),

                 mainPanel(
                   dataTableOutput('denseOutputPage')
                 )
               )
             )
           )
  ),
     
  tabPanel('Euromaidan Community',
           fluidPage(
             fluidPage(
               # Application title
               titlePanel("Topic Analysis by Hashtag"),
               sidebarLayout(
                 # Sidebar with a slider and selection inputs
                 sidebarPanel(
                   numericInput("tweetCount","Minimum Tweet Count:",0),
                   numericInput("followCount","Minimum Follower Count:",0),               
                   br(),
                   p('This page is designed to enable an analyst to identify topic clusters of interest by displaying the most frequent hashtags used by the clusters members over the past year.  First select a cluster, and then adjust the number of words and font size.  Additionally, you can select wether you would like to see the tags in their original language or translated.'),
                   img(src='logos.png',width=450)
                 ),
                 
                 mainPanel(
                   dataTableOutput('euromaidanOutputPage')
                 )
               )
             )
           )
  ),
  tabPanel('Definition of Attributes', 
           mainPanel(
             strong("ScreenName:"),
             p("hyperlink to the respective users\' Twitter account profile."),
             p(""),
             strong("CLUSTERING FILTERS: LOOK AT SETS OF USERS GROUPS BASED ON VARIOUS CLUSTERING ALGORITHMS"),
             p(""),
             strong("denseCluster:"),
             p("Only groups where the users have similar hashtag patterns as well as dense following and mention ties. d-0 means the user did not belong to any 'dense cluster'.  Most users will not, this technique merely returns the most highly connected groups within the network"),
             strong("topic_cluster:"),
             p("Similar to document clustering based on word distribution.  This clusters users based on hashtag use."),
             strong("RM_Louvain:"),
             p("Louvain groups run based on the undirected mention network."),
             strong("RF_Louvain:"),
             p("Louvain groups run based on the undirected following network."),
             p(""),
             strong("USER LEVEL METRICS"),
             p(""),
             strong("FiP:"),
             p("The proportion of the user's following ties contained in the detected OTGSC. 1 means that 100% of this user's following ties are present in this data."),
             strong("FoP:"),
             p("The proportion of the user's follower ties contained in the detected OTGSC. 1 means that 100% of this user's follower ties are present in this data."),
             strong("followingCount, followerCount: "),
             p(" refers to the users follower and following counts at the time of data collection from the Twitter API."),
             strong("urlRatio, mentionRatio: "),
             p(" mean number of urls or mentions per tweet."),
             strong("tweetCount: "),
             p(" refers to the users total number of posts at the time of data collection from the Twitter API."),
             strong("tweetsCollected: "),
             p(" refers to the number of tweets provided by the API at the time of data collection."),
             strong("lastTweet: "),
             p(" time of  at the time of data collection from the Twitter API."),
             p(""),
             strong("GEOTAGS BY USER BY COUNTRY"),
             p(""),
             strong("geoTag-country:"),
             p("Each field that starts with 'geoTag-' represents the number of posts by a given user tagged to a given country.  Less then .1% of posts within the dataset are geo-tagged."),
             strong("Under Construction: "),
             p(" ...")
             
             )
          )
              
  )
)

